"""
LRMC demo - sIRLSp

ECE 510

python version: 3.7.3

Spring 2019
"""

import numpy as np
import time
from numpy.linalg import norm
from scipy.linalg import fractional_matrix_power

def sIRLSp(X, mask, p):
    
    M, N = np.shape(X)
    print(f"shape of X:{np.shape(X)}")
    ## Algorithm parameters
    tol = 1e-8
    maxIter = 1000
    gammaMin = 1e-20
    eta = 1.1
    sn = norm(X)    # 2 - norm of X

    ## Initialize optimization variables
    Xhat = np.zeros([M, N])
    Xhat[mask] = X[mask]
    print(f"X[mask] shape:{np.shape(X[mask])}")
    gamma = 0.01 * sn**2
    s = gamma**(1 - p/2)

    for kk in range(maxIter):
        Xprev = Xhat
        W = fractional_matrix_power(Xhat.T @ Xhat + gamma * np.eye(N), p/2 -1)
        Xhat = Xhat - s * Xhat @ W
        Xhat = Xhat.real        # extract the real part of the 
        Xhat[mask] = X[mask]
        gamma = max(gamma/eta, gammaMin)
        s = gamma ** (1 - p/2)

        # Stopping conditions
        err = norm(Xprev - Xhat, ord='fro')/norm(Xhat, ord='fro')
        if err < tol:
            break

    return Xhat


